/*--------------------------------------------------------------------------
 *    
 *                         -- C N R S -- 
 *         Laboratoire d'Automatique et d'Analyse des Systemes 
 *                 7 Avenue du colonel Roche   
 *                     31 077 Toulouse Cedex
 *
 *  Auteur           : Sara Fleury
 *  Groupe           : Robotique et Intelligence Artificielle
 *  Nom du fichier   : piloRSTest.c
 *  Date de creation : 1995
 * 
 *  Description      : Fonction de test de la bibliotheque de lissage
 *                     des trajectoire type Reed & Shep.
 *
 *  Copyright (C) 1994 LAAS/CNRS.
 *----------------------------------------------------------------------*/

#ifndef VXWORKS
#  include <portLib.h>
#else
#  include <vxWorks.h>
#endif
#include <stdio.h>

#include "h2mathLib.h"
#include "piloCmdTaskFunc.h"
#include "piloStruct.h"
#include "piloConst.h"
#include "piloRSLibProto.h"
#include "piloSpeedLibProto.h"
#include "auto/piloType.h"

PILO_STR *piloDataStrId;
SMOOTH_TRAJ *piloSmoothTrajId=NULL;
static PILO_STR piloDataStr;
PILO_SPEED_REF_STR piloSpeedRefStr;

static void piloPrintTraj(int nbMvts);
static void piloPrintTraj2(int nbMvts);

#ifdef UNIX
main()
{
  int i;
  int bilan;
  int nbMvts;
  int nbMvtsF;
  int nbMvtsT;
  double var;
  double curv;
  double rayon = 1.;
  char typeMvt;
  /* VERIF*/
  double x, y, th, dth;

  piloDataStrId = &piloDataStr;

  /* Init */
  DATARS_INIT_POS_FLAG = TRUE;
  DATARS_X0 = 0.;
  DATARS_Y0 = 0.;
  DATARS_THETA0 = 0.;
  VMAX_REF = VMAX_PILO;
  WMAX_REF = WMAX_PILO;
  ALIN_REF = AMAX_LIN_PILO;
  AANG_REF = AMAX_ANG_PILO;

  while(1) {
    printf ("\nNb motions: ");
    scanf("%d", &DATARS_NB);

    for (i=0; i<DATARS_NB; i++) {
      printf ("curv dl : ");
      scanf ("%lf", &curv);
      scanf (" %lf", &var);
      if (curv == 0) {
	printf ("\t\t\t-> move %g m\n", var);
	DATARS_DTH(i) = 0.;
	DATARS_DL(i) = var;
      }
      if (fabs(curv) > 1000) {
	DATARS_DTH(i) = var;
	DATARS_DL(i) = 0.;
      }
      else {
	DATARS_DTH(i) = var*curv;
	DATARS_DL(i) = var; 
	printf ("\t\t\t-> turn %g rd %g m \n", DATARS_DTH(i), DATARS_DL(i));
      }
    }
     
    /*** END POS FOR CONTROL ***/
    x = DATARS_X0;
    y = DATARS_Y0;
    th = DATARS_THETA0;
    for (i=0; i<DATARS_NB; i++) {
      if (DATARS_DTH(i) == 0.) {
	x += DATARS_DL(i) * cos(th);
	y += DATARS_DL(i) * sin(th);
      }
      else {
	dth = DATARS_DTH(i);
	x += DATARS_DL(i)/DATARS_DTH(i) * (sin(th+dth) - sin(th));
	y += DATARS_DL(i)/DATARS_DTH(i) * (cos(th) - cos(th+dth));
	th += dth;
      }
    }
    
    /*-------------------------------------------------- 
     * Filtrage de la traj 
     */
    if ((bilan = piloRSFiltre(&nbMvts)) != OK) {
      printf ("piloRSFiltre error\n");
      break;
    }
    nbMvtsF = nbMvts;

    /***piloMvtsFreeAndAlloc (nbMvts);*/
    if (piloSmoothTrajId != NULL) free(piloSmoothTrajId);
    if ((piloSmoothTrajId = (SMOOTH_TRAJ *) malloc (nbMvts * sizeof(SMOOTH_TRAJ)))
	== NULL) {
      printf ("pilo: PB MALLOC\n");
      return (errnoGet());
    }
    memset((void *)piloSmoothTrajId, 0, nbMvts * sizeof(SMOOTH_TRAJ));
    
    X_START(0) = DATARS_X0;
    Y_START(0) = DATARS_Y0;
    THETA_START(0) = DATARS_THETA0;

    /*--------------------------------------------------
     * Determination des types de la trajectoire 
     */
    if ((bilan = piloRSTypes (&nbMvts)) != OK) {
      printf ("piloRSTypes error\n");
      break;
    }
    nbMvtsT = nbMvts;

    piloPrintTraj2(nbMvts);

    printf("** piloRSData  (%d mvt): END  x %7.4lf  y %7.4lf  th %7.4lf (%6.2lf deg)\n", 
	   nbMvtsF, x, y, th, th*180./PI);
    /*** VERIF TYPES ***/
    x = X_START(0);
    y = Y_START(0);
    th = THETA_START(0);
    for (i=0; i<nbMvtsT-1; i++) {
      if (TYPE_MVT(i) == SEGMENT) {
	x += VAR_LIN(i) * cos(th);
	y += VAR_LIN(i) * sin(th);
      }
      else if (TYPE_MVT(i) == TURN) {
	dth = VAR_ANG(i);
	x += RO_START(i) * (sin(th+dth) - sin(th));
	y += RO_START(i) * (cos(th) - cos(th+dth));
	th += dth;
      }
    }
    printf("** piloRSTypes (%d mvt): END  x %7.4lf  y %7.4lf  th %7.4lf (%6.2lf deg)\n", 
	   nbMvtsT, x, y, th, th*180./PI);


    /*--------------------------------------------------
     * Compute geometrical trajectory 
     */
    if (piloRSGeo (&nbMvts) != OK) {
      printf ("piloRSGeo error\n");
      break;
    }

    /*--------------------------------------------------
     * Compute velocities on trajectory 
     */
    V_START(0) = 0.;
    W_START(0) = 0.;
    if (piloSpeedProfiles (nbMvts) != OK) {
      printf ("piloSpeedProfiles error\n");
      break;
    }

    /*** VERIF GEO ***/
    printf("** piloRSGeo   (%d mvt): END  x %7.4lf  y %7.4lf  th %7.4lf (%6.2lf deg)\n", 
	   nbMvts, X_START(nbMvts-1), Y_START(nbMvts-1), 
	   THETA_START(nbMvts-1), 
	   THETA_START(nbMvts-1)*180./PI);


    piloPrintTraj(nbMvts);

    piloRSControlResult(nbMvts);
  } /* WHILE */

}
#endif

static void piloPrintTraj(int nbMvts) 
{
  int i;
  double time=0.;
  char tabType[] = "mtCAE";
  i = 0;
  printf("\n    seg  type\t  dist\t  ang\t  ro\t k \t vS\t wS\t t\t al\t aa\n");
  while (TYPE_MVT(i) != GOTO_END) {
    printf ("%d:  %2d    %c\t%6.3f\t%6.3f\t%6.3f\t%4.2f\t%4.2f\t%4.2f\t%6.4f\t%6.2f\t%6.2f\n", 
	    i, SEG_NUM(i), tabType[TYPE_MVT(i)], VAR_LIN(i),
	    VAR_ANG(i), RO_START(i), K_TRAJ(i), 
	    V_START(i), W_START(i), TIME_MIN(i), ACCEL_LIN(i), ACCEL_ANG(i));
    time += TIME_MIN(i);
    i++;
  }
  printf ("%d:  %2d    %c\t\t\t\t\t\t\t%6.4f\n", 
	  i, SEG_NUM(i), tabType[TYPE_MVT(i)], time);
}

static void piloPrintTraj2(int nbMvts) 
{
  int i;
  char tabType[] = "mtCAE";
  i = 0;
  printf("\n    seg  type\t  dist\t  ang\t  ro\t k\n");
  while (TYPE_MVT(i) != GOTO_END) {
    printf ("%d:  %2d    %c\t%6.3f\t%6.3f\t%6.3f\t%6.4f\n", 
	    i, SEG_NUM(i), tabType[TYPE_MVT(i)], VAR_LIN(i), 
	    VAR_ANG(i), RO_START(i),
	    K_TRAJ(i));
    i++;
  }
  printf ("%d:  %2d    %c\n", 
	  i, SEG_NUM(i), tabType[TYPE_MVT(i)]);
}
