#ifndef pilo_CONST_H
#define pilo_CONST_H


/* ------------------------------ H2 ------------------------------ */

#ifdef H2
#include "locoConst.h" 

#define LOCO_CMD_TASK_PERIOD 0.025
#define PILO_MOT_AXIS_WIDTH  LOCO_MOT_AXIS_WIDTH
#define PILO_DIST_TO_CONTROLED_POINT LOCO_DIST_TO_CONTROLED_POINT

/* Basic servo limits */
#define VMAX_ROBOT          LOCO_VMAX_ROBOT      /* 0.9 */
#define WMAX_ROBOT          LOCO_WMAX_ROBOT      /* 1 */
#define AMAX_LIN_ROBOT      LOCO_AMAX_LIN_ROBOT  /* 1 */
#define AMAX_ANG_ROBOT      LOCO_AMAX_ANG_ROBOT  /* 3 PI/2 = 4.71 */

/* Speed and accel limits for pilo
 * WARNINGS: must be less than servo limits to allow regulation
 */

#define VMAX_PILO          (VMAX_ROBOT * 0.95)      /*  .855 */
#define WMAX_PILO          (WMAX_ROBOT * 0.95)      /*  .95  */
#define AMAX_LIN_PILO      (AMAX_LIN_ROBOT * 0.4)   /*  .66 -> .4  */
#define AMAX_ANG_PILO      (AMAX_ANG_ROBOT * 0.3)   /* 2.356 -> 1.413 */

/* Accelerations max pour modifier dynamiquement les profils de vitesses.
 *
 * Elles doivent etre superieures a celles autorisees au client de pilo,
 * mais toujours inferieures a celle de loco. 
 * Pas de pb pour les vitesses car elles ne peuvent qu'etres diminuees.
 */
#define AMAX_LIN_PILO_MAJOR   (AMAX_LIN_ROBOT + AMAX_LIN_PILO)/2.   /*  .83 */
#define AMAX_ANG_PILO_MAJOR   (AMAX_ANG_ROBOT + AMAX_ANG_PILO)/2.   /* 3.53 */

#define PILO_SPEED_CORRECTION_V 1
#define PILO_SPEED_CORRECTION_W 1

#endif // H2

/* ------------------------------ locoDemo -------------------------- */
#ifdef LOCODEMO
#include "locoDemo/locoDemoConst.h"

#define LOCO_CMD_TASK_PERIOD 0.040
#define PILO_MOT_AXIS_WIDTH LOCODEMO_MOT_AXIS_WIDTH
#define PILO_DIST_TO_CONTROLED_POINT LOCODEMO_CNTR_POINT_DIST

#define VMAX_ROBOT LOCODEMO_VMAX
#define WMAX_ROBOT LOCODEMO_WMAX
#define AMAX_LIN_ROBOT LOCODEMO_ACCEL_LIN_MAX
#define AMAX_ANG_ROBOT LOCODEMO_ACCEL_ANG_MAX

#define PILO_SPEED_CORRECTION_V 1
#define PILO_SPEED_CORRECTION_W 1

/* Speed and accel limits for pilo
 * WARNINGS: must be less than servo limits to allow regulation
 */

#define VMAX_PILO          (VMAX_ROBOT * 0.95)      /*  .855 */
#define WMAX_PILO          (WMAX_ROBOT * 0.95)      /*  .95  */
#define AMAX_LIN_PILO      (AMAX_LIN_ROBOT * 0.4)   /*  .66 -> .4  */
#define AMAX_ANG_PILO      (AMAX_ANG_ROBOT * 0.3)   /* 2.356 -> 1.413 */

/* Accelerations max pour modifier dynamiquement les profils de vitesses.
 *
 * Elles doivent etre superieures a celles autorisees au client de pilo,
 * mais toujours inferieures a celle de jloco. 
 * Pas de pb pour les vitesses car elles ne peuvent qu'etres diminuees.
 */
#define AMAX_LIN_PILO_MAJOR   (AMAX_LIN_ROBOT + AMAX_LIN_PILO)/2.   /*  .83 */
#define AMAX_ANG_PILO_MAJOR   (AMAX_ANG_ROBOT + AMAX_ANG_PILO)/2.   /* 3.53 */

#endif /* LOCODEMO */

/* ------------------------------ JIDO ------------------------------ */

#ifdef JIDO
#include "jloco/jlocoConst.h"

#define LOCO_CMD_TASK_PERIOD 0.040
#define PILO_MOT_AXIS_WIDTH JLOCO_AXIS_WIDTH
#define PILO_DIST_TO_CONTROLED_POINT JLOCO_CNTR_POINT_DIST

/* Basic servo limits */
#define VMAX_ROBOT          JLOCO_VMAX_ROBOT      /* 0.9 */
#define WMAX_ROBOT          JLOCO_WMAX_ROBOT      /* 1 */
#define AMAX_LIN_ROBOT      JLOCO_AMAX_LIN_ROBOT  /* 1 */
#define AMAX_ANG_ROBOT      JLOCO_AMAX_ANG_ROBOT  /* PI/2 = 1.5 */

#define PILO_SPEED_CORRECTION_V 0.91
#define PILO_SPEED_CORRECTION_W 0.905

/* Speed and accel limits for pilo
 * WARNINGS: must be less than servo limits to allow regulation
 */

#define VMAX_PILO          (VMAX_ROBOT * 0.95)      /*  .855 */
#define WMAX_PILO          (WMAX_ROBOT * 0.95)      /*  .95  */
#define AMAX_LIN_PILO      (AMAX_LIN_ROBOT * 0.4)   /*  .66 -> .4  */
#define AMAX_ANG_PILO      (AMAX_ANG_ROBOT * 0.3)   /* 2.356 -> 1.413 */

/* Accelerations max pour modifier dynamiquement les profils de vitesses.
 *
 * Elles doivent etre superieures a celles autorisees au client de pilo,
 * mais toujours inferieures a celle de jloco. 
 * Pas de pb pour les vitesses car elles ne peuvent qu'etres diminuees.
 */
#define AMAX_LIN_PILO_MAJOR   (AMAX_LIN_ROBOT + AMAX_LIN_PILO)/2.   /*  .83 */
#define AMAX_ANG_PILO_MAJOR   (AMAX_ANG_ROBOT + AMAX_ANG_PILO)/2.   /* 3.53 */

#endif // JIDO


/* ------------------------------ XR4000 ------------------------------ */

#ifdef XR4000
#include "xr4000/xr4000Const.h"

#define LOCO_CMD_TASK_PERIOD 0.040
#define PILO_MOT_AXIS_WIDTH  XR4000_AXIS_WIDTH
#define PILO_DIST_TO_CONTROLED_POINT XR4000_DIST_TO_CONTROLED_POINT

/* Basic servo limits */
#define VMAX_ROBOT          XR4000_VMAX_ROBOT      /* 0.9 */
#define WMAX_ROBOT          XR4000_WMAX_ROBOT      /* 1 */
#define AMAX_LIN_ROBOT      XR4000_AMAX_LIN_ROBOT  /* 1 */
#define AMAX_ANG_ROBOT      XR4000_AMAX_ANG_ROBOT  /* PI/2 = 1.5 */

/* Speed and accel limits for pilo
 * WARNINGS: must be less than servo limits to allow regulation
 */

#define VMAX_PILO          (VMAX_ROBOT * 0.95)      /*  .855 */
#define WMAX_PILO          (WMAX_ROBOT * 0.95)      /*  .95  */
#define AMAX_LIN_PILO      (AMAX_LIN_ROBOT * 0.4)   /*  .66 -> .4  */
#define AMAX_ANG_PILO      (AMAX_ANG_ROBOT * 0.3)   /* 2.356 -> 1.413 */

/* Accelerations max pour modifier dynamiquement les profils de vitesses.
 *
 * Elles doivent etre superieures a celles autorisees au client de pilo,
 * mais toujours inferieures a celle de loco. 
 * Pas de pb pour les vitesses car elles ne peuvent qu'etres diminuees.
 */
#define AMAX_LIN_PILO_MAJOR   (AMAX_LIN_ROBOT + AMAX_LIN_PILO)/2.   /*  .83 */
#define AMAX_ANG_PILO_MAJOR   (AMAX_ANG_ROBOT + AMAX_ANG_PILO)/2.   /* 3.53 */

#define PILO_SPEED_CORRECTION_V 1
#define PILO_SPEED_CORRECTION_W 1

#endif // XR4000

/* ----------------------------- ATRV, B21R  ------------------------------ */

#if defined(ATRV) || defined(B21R)

#include "rflexConst.h"

#define PILO_MOT_AXIS_WIDTH RFLEX_AXIS_WIDTH
#define LOCO_CMD_TASK_PERIOD 0.040
#define PILO_DIST_TO_CONTROLED_POINT RFLEX_DIST_TO_CONTROLED_POINT

/* Basic servo limits */
#define VMAX_ROBOT          RFLEX_VMAX_ROBOT      /*  */
#define WMAX_ROBOT          RFLEX_WMAX_ROBOT      /*  */
#define AMAX_LIN_ROBOT      RFLEX_AMAX_LIN_ROBOT  /*  */
#define AMAX_ANG_ROBOT      RFLEX_AMAX_ANG_ROBOT  /*  */

/* Speed and accel limits for pilo
 * WARNINGS: must be less than servo limits to allow regulation
 */

#define VMAX_PILO          (VMAX_ROBOT * 0.95)      /*  .855 */
#define WMAX_PILO          (WMAX_ROBOT * 0.95)      /*  .95  */
#define AMAX_LIN_PILO      (AMAX_LIN_ROBOT * 0.4)   /*  .66 -> .4  */
#define AMAX_ANG_PILO      (AMAX_ANG_ROBOT * 0.3)   /* 2.356 -> 1.413 */
#define AMAX_LIN_PILO_MAJOR   (AMAX_LIN_ROBOT + AMAX_LIN_PILO)/2.   /*  .83 */
#define AMAX_ANG_PILO_MAJOR   (AMAX_ANG_ROBOT + AMAX_ANG_PILO)/2.   /* 3.53 */

#define PILO_SPEED_CORRECTION_V 1
#define PILO_SPEED_CORRECTION_W 1


#endif // defined(ATRV) || defined(B21R)

/* ------------------------------ FAKE ------------------------------ */

#ifdef FAKE

#define LOCO_CMD_TASK_PERIOD 0.040
#define PILO_MOT_AXIS_WIDTH 1.0
#define PILO_DIST_TO_CONTROLED_POINT 0.6

/* Basic servo limits */
#define VMAX_ROBOT          0.9
#define WMAX_ROBOT          1
#define AMAX_LIN_ROBOT      1
#define AMAX_ANG_ROBOT      1.5

/* Speed and accel limits for pilo
 * WARNINGS: must be less than servo limits to allow regulation
 */

#define VMAX_PILO          (VMAX_ROBOT * 0.95)      /*  .855 */
#define WMAX_PILO          (WMAX_ROBOT * 0.95)      /*  .95  */
#define AMAX_LIN_PILO      (AMAX_LIN_ROBOT * 0.4)   /*  .66 -> .4  */
#define AMAX_ANG_PILO      (AMAX_ANG_ROBOT * 0.3)   /* 2.356 -> 1.413 */

/* Accelerations max pour modifier dynamiquement les profils de vitesses.
 *
 * Elles doivent etre superieures a celles autorisees au client de pilo,
 * mais toujours inferieures a celle de jloco. 
 * Pas de pb pour les vitesses car elles ne peuvent qu'etres diminuees.
 */
#define AMAX_LIN_PILO_MAJOR   (AMAX_LIN_ROBOT + AMAX_LIN_PILO)/2.   /*  .83 */
#define AMAX_ANG_PILO_MAJOR   (AMAX_ANG_ROBOT + AMAX_ANG_PILO)/2.   /* 3.53 */

#define PILO_SPEED_CORRECTION_V 1
#define PILO_SPEED_CORRECTION_W 1

#endif // FAKE

/* ---------------------------------------------------------------------- */
/* Les types de mouvement pendant une execution */
typedef enum {SEGMENT,            /* Segment de droite */
	      TURN,               /* Arc de cercle */
	      CLOTHO,             /* Arc de clotho */
	      ANTI_CLOTHO,        /* Arc d'anti-clotho */
	      GOTO_END            /* Dernier point de la traj */
}PILO_TYPE_MVT;

#endif // pilo_CONST_H
